/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __FSTP_HEADER_INCLUDED__
#define __FSTP_HEADER_INCLUDED__

#include <g_globals.h>

typedef enum fstp_compress{
	FSTP_COMPRESS_NONE,
	FSTP_COMPRESS_ZLIB,
	FSTP_COMPRESS_BZIP,
	FSTP_COMPRESS_MAX
}fstp_compress_t;

typedef enum fstp_type{
	FSTP_SEG_PADDING,
	FSTP_SEG_PACKET,
	FSTP_SEG_COMMAND,
	FSTP_SEG_TIMESTAMP,
	FSTP_SEG_EOT,
	FSTP_SEG_MAX
}fstp_type_t;

/* We travel in UDP so quite small */
#define FSTP_MAX_PAYLOAD	1024
#define FSTP_CANARY		0xDEADD00D

/* No magic numbers !! */
typedef struct fstp_hdr{
	u_int16_t	hdr_len;
	u_int8_t	compression;	/* Compression mode */
	u_int8_t	level;		/* Compression level */
	u_int32_t	len;		/* Length of payload */
	u_int32_t	csum;		/* Checksum of whole packet */
}fstp_hdr_t, *p_fstp_hdr;

/* Header for each data segment */
typedef struct segment_hdr {
	u_int32_t	type;
	u_int32_t	caplen;		/* Length of data present */
}segment_hdr_t, *p_segment_hdr;

/* An FSTP packet */
typedef struct fstp_pkt {
	fstp_hdr_t	hdr;		/* The header */
	char		data[FSTP_MAX_PAYLOAD];
	u_int32_t	canary;	
}fstp_pkt_t, *p_fstp_pkt;

#define FSTP_EOT_SIZE		(sizeof(segment_hdr_t))
#define FSTP_HDR_SIZE		(sizeof(fstp_hdr_t))
#define FSTP_SEG_SIZE		(sizeof(segment_hdr_t))

#endif //__FSTP_HEADER_INCLUDED__
