/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __MATCHER_HEADER_INCLUDED__
#define __MATCHER_HEADER_INCLUDED__

#include <s_globals.h>
#include <lists.h>
#include <targets.h>

/* Takes a parsed token and creates a private struct  */
typedef void * (*proc_match_validate)(char *, p_kv);

/* Cleans up said parsed token when rule is unloaded */
typedef void   (*proc_match_cleanup)(void *);

/* Match a packet agains private data  */
typedef int (*proc_match_match)(void *, p_packet, int);

/* A matcher */
typedef struct matcher {
	list_head_t		l;
	char			*name;
	proc_match_validate	validate;
	proc_match_cleanup	cleanup;
	proc_match_match	match;
}matcher_t, *p_matcher;

/* Rule element */
typedef struct relement {
	list_head_t		l;
	void			*priv; /* cleanup with match->cleanup() */
	p_matcher		match;
	int			negate;
}relement_t, *p_relement;

/* A rule object: 
* All criterion in a rule must be matched 
* positively. When matched, the packet is sent
* to the relevent target
*/
typedef struct rule {
	list_head_t		l;

	/* List of criteria to match */
	p_relement		criteria;

	p_target		target;
	void			*t_priv; /* cleanup with target->cleanup */
}rule_t, *p_rule;

/* Function declares */
int matcher_init(void);
void matcher_cleanup(int code, void *priv);
p_matcher matcher_get(char *);
void matcher_begin(p_packet);

/* Language plugins use these */
#ifndef __PLUGIN__
int matcher_add_rule(p_rule);
int matcher_add_criteria_to_rule(p_rule, char *, char *, p_kv, int);
int matcher_set_rule_target(p_rule r, char *target, char *val);
#endif

typedef int (*proc_add_rule)(p_rule);
typedef int (*proc_add_criteria)(p_rule, char *, char *, p_kv, int);
typedef int (*proc_set_target)(p_rule, char *, char *);

/* Function that plugins call to add new 
* matchers */
typedef int (*proc_matcher_add)(char *, 
		proc_match_validate,
		proc_match_cleanup,
		proc_match_match);
typedef void (*proc_rule_cleanup)(p_rule);

#ifndef __PLUGIN__
int matcher_add (char *, proc_match_validate,
	       proc_match_cleanup,
       	       proc_match_match);
void matcher_rule_cleanup(p_rule r);
#endif


#define SAFE_INIT_RULE_PTR(r) \
	r->target = NULL; \
	r->t_priv = NULL; \
	r->criteria = NULL; \
	r->l.next = NULL; \
	r->l.prev = NULL;

#define MATCH_RET_POSITIVE return (negate) ? 0 : 1;
#define MATCH_RET_NEGATIVE return (negate) ? 1 : 0;

#endif //__MATCHER_HEADER_INCLUDED__
